########################################################################
# latex.mk
#
# Make production of LaTeX documents trivial, not just easy.
#
####################################
# Usage.
#
# In order to use this, simply define a variable PRODUCTS to specify
# your final documents, like:
#
#   PRODUCTS := mydoc1.pdf mydoc2.pdf
#
# Optionally, you may set the variable PDFMODE, which defaults to,
# "-pdf", to something else, such as, "-xelatex" or, "-luatex."
#
# Followed by the include of this fragment:
#
#   include <relative-path>/latex.mk
#
# Optionally, following the include you may:
#
#   override clean_dirs += more dirs to clean
#
#   override clean_files += more files to clean
#
# See below for the default contents of these variables. Other
# override-able variables are described in the customizable section
# below.
#
####################################
# Defined targets.
#
# An unqualified "make" is equivalent to, "make all," which will produce
# all defined $(PRODUCTS) unless you define the ALL variable to be
# (presumably) some subset of $(PRODUCTS).
#
# Other targets:
#
#   clean: Remove $(clean_dirs) and generated files (according to
#          latexmk).
#
#   clobber: Additionally remove $(products)
#
#   macenv: Add the LaTeX environment variables (TEXINPUTS, etc.) to the
#           Mac OS X GUI environment to enable applications such as
#           aquamacs started from the GUI to have the correct settings
#           for (e.g.) internal LaTeX invocation.
#
#   view: Start the correctly defined viewer for each defined
#         $(PRODUCT).  e.g. to set the pdf view to skim, you need a
#         file ~/.latexmkrc containing:
#
#           $pdf_previewer = "open -a skim %O %S"
#
#   <prod>: Make the single product <prod> regardless of whether it is
#           in $(PRODUCTS).
#
#   view-<prod>: Run the defined viewer for <prod> after first making
#                sure that it is up-to-date.
#
####################################
# Advanced use:
#
# If you need to add targets (generation of some dependencies for
# example, like figures), you should define that target, eg:
#
# myFigure.tex: myFigure.dat
#
# The latexmk machinery should take care of ensuring that it is invoked
# at the correct time.
#		
########################################################################

########################################################################
# These variables may be overridden or added to.
########################################################################

# Default way to generate PDF.
PDFMODE = -pdf
# Extra command-line options.
LATEXMK_EXTRA_OPTS =
# Extra perl directives for LATEXMK.
LATEXMK_EXTRA_CMDS =

# Files removed by, "make tidy."
tidy_files =  \
  *.bak \
  *.orig \
  *.rej \
  *.old \
  *~ \
	*~[0-9]

# Extra directories to make sure we empty.
clean_dirs = $(DEPS_DIR)

# Extra files to make sure we remove.
clean_files =

########################################################################
# No user-serviceable parts below.
########################################################################

####################################
# Define "all" target early.

all: $(if $(ALL),$(ALL),$(PRODUCTS))

####################################
# Variable definitions.

DEPS_DIR = .deps

LATEXMK = latexmk -f- \
            -recorder -use-make -deps \
            $(LATEXMK_EXTRA_OPTS) \
            -e 'push @generated_exts, qw(nav snm vrb);' \
            -e '$(LATEXMK_EXTRA_CMDS)'

ensure_deps = mkdir -p $(DEPS_DIR)

safe_fail = { $(RM) "$@"; false; }

def_maybe_pushenv = \
function maybe_pushenv() \
{ \
  local var; \
  for var in "$$@"; do \
    if [[ -n "$$var" ]] && [[ -n "$$(eval echo \"\$$$$var\")" ]] ; then \
      launchctl setenv "$$var" "$$(eval echo \"\$$$$var\")" && \
        echo "Pushed $$var to application env."; \
    else \
      true; \
    fi; \
  done; \
}

####################################
# Dependencies.

$(foreach file,$(PRODUCTS),$(eval -include $(DEPS_DIR)/$(file).d))

####################################
# Targets

.PHONY: clean clobber macenv tidy cleanit cleandirs cleanfiles

$(filter %.dvi, $(PRODUCTS)): %.dvi : %.tex
	$(ensure_deps)
	$(LATEXMK) -dvi -deps-out=$(DEPS_DIR)/$@.d $(<) || $(safe_fail)

$(filter %.pdf, $(PRODUCTS)): %.pdf : %.tex
	$(ensure_deps)
	$(LATEXMK) $(PDFMODE) -deps-out=$(DEPS_DIR)/$@.d $(<) || $(safe_fail)

$(filter %.ps, $(PRODUCTS)): %.ps : %.tex
	$(ensure_deps)
	$(LATEXMK) -ps -deps-out=$(DEPS_DIR)/$@.d $(<) || $(safe_fail)

cleanit: cleandirs cleanfiles

cleandirs:
	$(RM) -r $(clean_dirs)

cleanfiles:
	$(LATEXMK) -use-make- $(clean_arg) -deps-
	$(RM) $(clean_files)

clean: clean_arg = -c
clean: cleanit

tidy:
	$(RM) $(tidy_files)

clobber: clean_arg = -C
clobber: cleanit

macenv:
	@$(def_maybe_pushenv); maybe_pushenv TEXINPUTS BSTINPUTS BIBINPUTS MFINPUTS MPINPUTS

# View targets
.PHONY: $(foreach d, $(PRODUCTS), view-$d) view

$(filter %.pdf,$(foreach d,$(PRODUCTS),view-$d)) : view-% : %
	latexmk $(PDFMODE) -pv $(*:.pdf=)

$(filter %.dvi,$(foreach d,$(PRODUCTS),view-$d)) : view-% : %
	latexmk -dvi -pv $(*:.dvi=)

$(filter %.ps,$(foreach d,$(PRODUCTS),view-$d))  : view-% : %
	latexmk -ps -pv $(*:.ps=)

view: $(foreach v,$(filter %.ps %.dvi %.pdf,$(PRODUCTS)),view-$(v))

### Local Variables:
### mode: makefile-gmake
### End:
